/*
 *  g_lucid_context.c
 *
 *  Copyright (c) 2005 The Regents of the University of Michigan.
 *  All rights reserved.
 *
 *  Andy Adamson <andros@umich.edu>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "mglueP.h"

OM_uint32
gss_export_lucid_sec_context(OM_uint32 *minor_status,
			     gss_ctx_id_t *context_handle,
			     OM_uint32 version, void **internal_buffer)
{
    OM_uint32       status;
    gss_union_ctx_id_t ctx;
    gss_mechanism   mech;

    GSS_INITIALIZE;

    if (context_handle == GSS_C_NO_CONTEXT)
	return GSS_S_NO_CONTEXT;

    ctx = (gss_union_ctx_id_t) * context_handle;
    mech = __gss_get_mechanism(ctx->mech_type);

    if (mech) {

	if (mech->gss_export_lucid_sec_context)
	    status = mech->gss_export_lucid_sec_context(minor_status,
							&ctx->internal_ctx_id,
							version,
							internal_buffer);
	else
	    status = GSS_S_BAD_BINDINGS;

	return (status);
    }

    return (GSS_S_NO_CONTEXT);
}

OM_uint32
gss_free_lucid_sec_context(OM_uint32 *minor_status,
			   gss_ctx_id_t context_handle,
			   void *internal_buffer)
{
    OM_uint32       status;
    gss_union_ctx_id_t ctx;
    gss_mechanism   mech;

    GSS_INITIALIZE;

    if (context_handle == GSS_C_NO_CONTEXT)
	return GSS_S_NO_CONTEXT;

    ctx = (gss_union_ctx_id_t) context_handle;
    mech = __gss_get_mechanism(ctx->mech_type);

    if (mech) {

	if (mech->gss_free_lucid_sec_context)
	    status = mech->gss_free_lucid_sec_context(minor_status,
						      internal_buffer);
	else
	    status = GSS_S_BAD_BINDINGS;

	return (status);
    }

    return (GSS_S_NO_CONTEXT);
}
